# SPDX-FileCopyrightText: 2021 Jose David
#
# SPDX-License-Identifier: MIT
"""
Creates a bitmap color picker of 100 pixels
"""

from color_picker.wheel_maker import make_wheel

make_wheel("wheel100.bmp", 100, 0x000000)
